//-----------------------------------------------------------------------------
#include "AssignIPDlg.h"
#include "wx/button.h"
#include "wx/combobox.h"
#include "wx/socket.h"
#include "wx/textctrl.h"
#include "wx/textdlg.h"
#include "wxAbstraction.h"

//=============================================================================
//================== Implementation LogOutputHandlerDlg =======================
//=============================================================================

BEGIN_EVENT_TABLE(AssignIPDlg, wxDialog)
	EVT_BUTTON(widBtnBuildMACAddress, AssignIPDlg::OnBtnBuildMACAddress)
	EVT_BUTTON(widBtnExecute, AssignIPDlg::OnBtnExecute)
END_EVENT_TABLE()

//-----------------------------------------------------------------------------
AssignIPDlg::AssignIPDlg( wxWindow* pParent, wxDynamicLibrary& TLILib )
 : wxDialog(pParent, wxID_ANY, wxString(wxT("Assign IP address to a device")), wxDefaultPosition,
   wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX | wxMINIMIZE_BOX),
   TLILib_(TLILib)
//-----------------------------------------------------------------------------
{
	pTLOpen_ = (TTLOpen)TLILib_.GetSymbol( wxT("TLOpen") );
	pTLClose_ = (TTLClose)TLILib_.GetSymbol( wxT("TLClose") );
	pTLGetNumInterfaces_ = (TTLGetNumInterfaces)TLILib_.GetSymbol( wxT("TLGetNumInterfaces") );
	pTLGetInterfaceName_ = (TTLGetInterfaceName)TLILib_.GetSymbol( wxT("TLGetInterfaceName") );
	pTLIMV_MACFromSerial_ = (TTLIMV_MACFromSerial)TLILib_.GetSymbol( wxT("TLIMV_MACFromSerial" ) );
	pTLIMV_ForceIP_ = (TTLIMV_ForceIP)TLILib_.GetSymbol( wxT("TLIMV_ForceIP" ) );

	/*
		|-------------------------------------|
		| pTopDownSizer                       |
		| |---------------------------------| |
		| |          spacer                 | |
		| | pGroupBoxSizer                  | |
		| | |-----------------------------| | |
		| | | pEditElementsGridSizer      | | |
		| | | |-------------------------| | | |
		| | | |                         | | | |
		| | | |            | pSizer   | | | | |
		| | | |                         | | | |
		| | | |-------------------------| | | |
		| | |-----------------------------| | |
		| |          spacer                 | |
		| |                    pBtnExecute_ | |
		| |---------------------------------| |
		|-------------------------------------|
	*/

	wxBoxSizer* pTopDownSizer = new wxBoxSizer(wxVERTICAL);
	wxPanel* pPanel = new wxPanel(this);

	pTopDownSizer->AddSpacer( 10 );

	wxBoxSizer* pGroupBoxSizer = new wxStaticBoxSizer(wxVERTICAL, pPanel, wxT("Parameters:"));
	wxFlexGridSizer* pEditElementsGridSizer = new wxFlexGridSizer(2, 5);
	pEditElementsGridSizer->AddGrowableCol( 1, 3 );

	// row 1
	pEditElementsGridSizer->Add( new wxStaticText(pPanel, wxID_ANY, wxT("Device MAC address:")), wxSizerFlags().Left().Center() );
	wxBoxSizer* pSizer = new wxBoxSizer(wxHORIZONTAL);
	pTCDeviceMACAddress_ = new wxTextCtrl( pPanel, widTCMACAddress );
	pBtnBuildMACAddress_ = new wxButton(pPanel, widBtnBuildMACAddress, wxT("&Build MAC address"));
	pBtnBuildMACAddress_->SetToolTip( wxT("If you don't know the MAC address of your MATRIX VISION device\n press this button to construct it from the device serial.") );
	pSizer->Add( pTCDeviceMACAddress_, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	pSizer->Add( pBtnBuildMACAddress_, wxSizerFlags().Right() );
	pEditElementsGridSizer->Add( pSizer, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 2
	pEditElementsGridSizer->Add( new wxStaticText(pPanel, wxID_ANY, wxT("Desired IP address:")), wxSizerFlags().Left().Center() );
	pTCDesiredIPAddress_ = new wxTextCtrl(pPanel, widDesiredIPAddress);
	pEditElementsGridSizer->Add( pTCDesiredIPAddress_, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 3
	pEditElementsGridSizer->Add( new wxStaticText(pPanel, wxID_ANY, wxT("Desired subnet mask:")), wxSizerFlags().Left().Center() );
	pTCDesiredSubnetMask_ = new wxTextCtrl(pPanel, widDesiredSubnetMask, wxT("255.255.255.0"));
	pEditElementsGridSizer->Add( pTCDesiredSubnetMask_, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	// row 4
	pEditElementsGridSizer->Add( new wxStaticText(pPanel, wxID_ANY, wxT("Desired gateway:")), wxSizerFlags().Left().Center() );
	pTCDesiredGateway_ = new wxTextCtrl(pPanel, widDesiredGateway);
	pEditElementsGridSizer->Add( pTCDesiredGateway_, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	pGroupBoxSizer->Add( pEditElementsGridSizer, wxSizerFlags().Align( wxGROW ) );
	// row 5
	pEditElementsGridSizer->Add( new wxStaticText(pPanel, wxID_ANY, wxT("Local adapter used for sending:")), wxSizerFlags().Left().Center() );
	pCBAdapterList = new wxComboBox(pPanel, widAdapterList, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, 0, wxCB_DROPDOWN);
	pEditElementsGridSizer->Add( pCBAdapterList, wxSizerFlags(2).Align( wxGROW | wxALIGN_CENTER_VERTICAL ) );
	if( !pTLOpen_ || !pTLClose_ || !pTLGetNumInterfaces_ || !pTLGetInterfaceName_ || 
		!pTLIMV_MACFromSerial_ || !pTLIMV_ForceIP_ )
	{
		wxMessageBox(wxT("A more recent driver package is needed."), wxT("Driver update needed"), wxOK | wxICON_INFORMATION, this);
	}
	else
	{
		int result = pTLOpen_( &hTL_ );
		unsigned int cnt = 0;
		result = pTLGetNumInterfaces_( hTL_, &cnt );
		if( result == 0 )
		{
			const size_t MAX_IP_ADDRESS_LENGTH = 16;
			char buf[MAX_IP_ADDRESS_LENGTH];
			for( unsigned int i=0; i<cnt; i++ )
			{
				size_t bufSize = MAX_IP_ADDRESS_LENGTH;
				if( pTLGetInterfaceName_( hTL_, i, buf, &bufSize ) == 0 )
				{
					pCBAdapterList->AppendString( ConvertedString(buf) );
				}
			}
		}
		if( pCBAdapterList->GetCount() )
		{
			pCBAdapterList->Select( 0 );
		}
	}

	pTopDownSizer->Add( pGroupBoxSizer, wxSizerFlags().Expand() );
	pTopDownSizer->AddSpacer( 10 );
	pBtnExecute_ = new wxButton(pPanel, widBtnExecute, wxT("&Execute"));
	pTopDownSizer->Add( pBtnExecute_, wxSizerFlags().Right() );
	pPanel->SetSizer( pTopDownSizer );
	pTopDownSizer->SetSizeHints( this );
}

//-----------------------------------------------------------------------------
AssignIPDlg::~AssignIPDlg()
//-----------------------------------------------------------------------------
{
	if( pTLClose_ && hTL_ )
	{
		pTLClose_( hTL_ );
	}
}

//-----------------------------------------------------------------------------
void AssignIPDlg::OnBtnBuildMACAddress( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	wxTextEntryDialog dlg(this,
		wxT("Please note that only serial numbers of MATRIX VISION GmbH\n")
		wxT("devices can be converted using this dialog"),
		wxT("Please enter the serial number of the device"),
		wxT(""),
		wxOK | wxCANCEL);

	if (dlg.ShowModal() == wxID_OK)
	{
		if( !pTLIMV_MACFromSerial_ )
		{
			wxMessageBox(wxT("A more recent driver package is needed."), wxT("Driver update needed"), wxOK | wxICON_INFORMATION, this);
			return;
		}

		size_t bufSize = 0;
		int result = pTLIMV_MACFromSerial_( dlg.GetValue().mb_str(), 0, &bufSize );
		if( result != 0 )
		{
			wxMessageBox(wxT("Failed to query the result size for a string buffer. Either the serial number\nEntered was invalid or another problem has been detected.\nMore information can be found in the log files"), wxT("ERROR"), wxOK | wxICON_INFORMATION, this);
			return;
		}

		char* pBuf = new char[bufSize];
		memset( pBuf, 0, bufSize );
		result = pTLIMV_MACFromSerial_( dlg.GetValue().mb_str(), pBuf, &bufSize );
		pTCDeviceMACAddress_->SetValue( ConvertedString(pBuf) );
		delete [] pBuf;
		if( result != 0 )
		{
			wxMessageBox(wxT("Failed to convert serial number\nMore information can be found in the log files"), wxT("ERROR"), wxOK | wxICON_INFORMATION, this);
			return;
		}
	}
}

//-----------------------------------------------------------------------------
void AssignIPDlg::OnBtnExecute( wxCommandEvent& )
//-----------------------------------------------------------------------------
{
	if( !pTLIMV_ForceIP_ )
	{
		wxMessageBox(wxT("A more recent driver package is needed."), wxT("Driver update needed"), wxOK | wxICON_INFORMATION, this);
		return;
	}

	wxString adapter(( pCBAdapterList->GetCount() > 0 ) ? pCBAdapterList->GetValue() : wxString(wxEmptyString));
	int result = pTLIMV_ForceIP_( pTCDeviceMACAddress_->GetValue().mb_str(),
		pTCDesiredIPAddress_->GetValue().mb_str(),
		pTCDesiredSubnetMask_->GetValue().mb_str(),
		pTCDesiredGateway_->GetValue().mb_str(),
		adapter.mb_str(),
		1000 );

	if( result != 0 )
	{
		wxMessageBox(wxString::Format( wxT("The execution of the command failed(error code: 0x%x), More information can be found in the log file."), result ), wxT("Command execution failed"), wxOK | wxICON_INFORMATION, this);
	}
}

