//-----------------------------------------------------------------------------
#ifndef imageCanvasH
#define imageCanvasH imageCanvasH
//-----------------------------------------------------------------------------
#include "DrawingCanvas.h"
#include <set>
#include <vector>
#include <wx/bitmap.h>
#include <wx/brush.h>
#include <wx/colour.h>
#include <wx/dcclient.h>
#include <wx/image.h>

typedef std::set<AOI*> AOISet;

struct ImageCanvasImpl;

//-----------------------------------------------------------------------------
class ImageCanvas : public DrawingCanvas
//-----------------------------------------------------------------------------
{
public:
	ImageCanvas() {}
	ImageCanvas( wxWindow* pApp, wxWindow* parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
				 long style = wxSUNKEN_BORDER, const wxString& name = wxT("ImageCanvas"), bool boActive = true );
	~ImageCanvas();
	void									OnPaint( wxPaintEvent & );
	bool									IsScaled( void ) const { return m_boScaling; }
	bool									OverlayMessagesActive( void ) const { return m_boShowOverlayMessages; }
	bool									InfoOverlayActive( void ) const { return m_boShowInfoOverlay; }
	AOI*									RegisterAOI( const wxRect&, const wxColour&, const wxString& );
	bool									RemoveAOI( AOI* pAOI );
	//-----------------------------------------------------------------------------
	enum TSaveResult
	//-----------------------------------------------------------------------------
	{
		srOK,
		srNoImage,
		srFailedToSave
	};
	TSaveResult								SaveCurrentImage( const wxString& filenameAndPath, const wxString& extension ) const;
	bool									SetAOI( AOI* pAOI, int x, int y, int w, int h );
	bool									SetImage( const mvIMPACT::acquire::ImageBuffer* pIB );
	void									SetScaling( bool boOn );
	void									SetInfoOverlay( const std::vector<wxString>& infoStrings );
	void									SetInfoOverlayMode( bool boOn );
	void									SetPerformanceWarningOutput( bool boOn );
private:
	ImageCanvasImpl*						m_pImpl;
	AOISet									m_aoiSet;
	bool									m_boScaling;
	bool									m_boShowInfoOverlay;
	std::vector<wxString>					m_infoStringsOverlay;
	bool									m_boShowOverlayMessages;
	wxEvtHandler*							m_pAppEventHandler;
	int										m_pAppWinID;
	const mvIMPACT::acquire::ImageBuffer*	m_pIB;
	void									BlitAOIs( wxPaintDC& dc, double scaleFactor, int bmpXOff, int bmpYOff, int bmpW, int bmpH );
	void									BlitInfoStrings( wxPaintDC& dc, double scaleFactor, int bmpXOff, int bmpYOff, int bmpW, int bmpH );
	void									BlitPerformanceMessages( wxPaintDC& dc, int bmpXOff, int bmpYOff, TImageBufferPixelFormat pixelFormat );
	void									DeleteAOIs( void );
	TSaveResult								StoreImage( const wxImage& img, const wxString& filenameAndPath, const wxString& extension ) const;
	DECLARE_DYNAMIC_CLASS(ImageCanvas)
	DECLARE_EVENT_TABLE()
	void									OnDblClick( wxMouseEvent& );
};

#endif // imageCanvasH
